@@ -6,6 +6,8 @@ use super::tools;
 use super::*;
 pub mod build;
 use build::*;
+pub mod handler;
+use handler::*;
 
 ///Panics if a disconnect is detected between all colfind methods.
 pub fn assert_query<T: Aabb>(bots: &mut [T]) {
@@ -51,6 +53,53 @@ pub fn assert_query<T: Aabb>(bots: &mut [T]) {
     assert_eq!(naive_res, nosort_res);
 }
 
+#[cfg(feature = "rayon")]
+pub mod par {
+    use super::*;
+    pub trait ParCollidingPairsApi<T: Aabb> {
+        fn par_colliding_pairs(
+            &mut self,
+            func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>) + Clone + Send,
+        );
+    }
+
+    impl<'a, T: Aabb + Send> ParCollidingPairsApi<T> for TreeInner<Node<'a, T>, DefaultSorter>
+    where
+        T::Num: Send,
+    {
+        fn par_colliding_pairs(
+            &mut self,
+            func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>) + Clone + Send,
+        ) {
+            CollidingPairsBuilder::new(self, DefaultNodeHandler::new(func)).build_par();
+        }
+    }
+
+    impl<'a, T: Aabb + Send> ParCollidingPairsApi<T> for TreeInner<Node<'a, T>, NoSorter>
+    where
+        T::Num: Send,
+    {
+        fn par_colliding_pairs(
+            &mut self,
+            func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>) + Clone + Send,
+        ) {
+            CollidingPairsBuilder::new(self, NoSortNodeHandler::new(func)).build_par();
+        }
+    }
+
+    impl<'a, T: Aabb + Send> ParCollidingPairsApi<T> for SweepAndPrune<'a, T>
+    where
+        T::Num: Send,
+    {
+        fn par_colliding_pairs(
+            &mut self,
+            func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>) + Clone + Send,
+        ) {
+            self.par_query(func);
+        }
+    }
+}
+
 ///
 /// Make colliding pair queries
 ///
@@ -69,27 +118,21 @@ impl<'a, T: Aabb> CollidingPairsApi<T> for AabbPin<&'a mut [T]> {
 
 use crate::tree::TreeInner;
 
-impl<'a, T: Aabb, S: NodeHandler<T>> CollidingPairsApi<T> for TreeInner<Node<'a, T>, S> {
+impl<'a, T: Aabb> CollidingPairsApi<T> for TreeInner<Node<'a, T>, DefaultSorter> {
     fn colliding_pairs(&mut self, func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>)) {
-        self.colliding_pairs_builder(func).build()
+        CollidingPairsBuilder::new(self, DefaultNodeHandler::new(func)).build()
+    }
+}
+
+impl<'a, T: Aabb> CollidingPairsApi<T> for TreeInner<Node<'a, T>, NoSorter> {
+    fn colliding_pairs(&mut self, func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>)) {
+        CollidingPairsBuilder::new(self, NoSortNodeHandler::new(func)).build()
     }
 }
 
 impl<'a, T: Aabb> CollidingPairsApi<T> for SweepAndPrune<'a, T> {
     fn colliding_pairs(&mut self, func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>)) {
-        ///Sweep and prune algorithm.
-        fn query_sweep_mut<T: Aabb>(
-            axis: impl Axis,
-            bots: &mut [T],
-            mut func: impl CollisionHandler<T>,
-        ) {
-            broccoli_tree::util::sweeper_update(axis, bots);
-
-            let mut prevec = PreVec::with_capacity(2048);
-            let bots = AabbPin::new(bots);
-            oned::find_2d(&mut prevec, axis, bots, &mut func, true);
-        }
-        query_sweep_mut(default_axis(), self.inner, func)
+        self.query(func);
     }
 }
 
@@ -100,187 +143,182 @@ pub struct SweepAndPrune<'a, T> {
     inner: &'a mut [T],
 }
 
-impl<'a, T> SweepAndPrune<'a, T> {
+impl<'a, T: Aabb> SweepAndPrune<'a, T> {
     pub fn new(inner: &'a mut [T]) -> Self {
+        let axis = default_axis();
+        broccoli_tree::util::sweeper_update(axis, inner);
+
         SweepAndPrune { inner }
     }
+
+    pub fn query(&mut self, mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>)) {
+        let mut prevec = PreVec::with_capacity(2048);
+        let bots = AabbPin::from_mut(self.inner);
+        oned::find_2d(&mut prevec, default_axis(), bots, &mut func, true);
+    }
+
+    #[cfg(feature = "rayon")]
+    ///Sweep and prune algorithm.
+    pub fn par_query(
+        &mut self,
+        mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>) + Clone + Send,
+    ) where
+        T: Send,
+    {
+        let axis = default_axis();
+        let mut prevec = Vec::with_capacity(2048);
+        let bots = AabbPin::from_mut(self.inner);
+        let a2 = axis.next();
+        let _ = oned::find_par(
+            &mut prevec,
+            axis,
+            bots,
+            move |a: AabbPin<&mut T>, b: AabbPin<&mut T>| {
+                if a.get().get_range(a2).intersects(b.get().get_range(a2)) {
+                    func(a, b);
+                }
+            },
+        );
+    }
 }
 
 use crate::tree::splitter::Splitter;
 
-pub trait CollidingPairsBuilderApi<'a, T: Aabb, SO: NodeHandler<T>> {
-    fn vistr_mut<'b>(&'b mut self) -> VistrMutPin<'b, broccoli_tree::node::Node<'a, T>>;
-    fn sorter(&mut self) -> SO;
-
-    fn colliding_pairs_builder<'b, F>(
-        &'b mut self,
-        func: F,
-    ) -> CollidingPairsBuilder<'a, 'b, T, SO, F>
-    where
-        F: FnMut(AabbPin<&mut T>, AabbPin<&mut T>),
-    {
-        let sorter = self.sorter();
-        CollidingPairsBuilder {
-            vis: CollVis::new(self.vistr_mut(), default_axis().to_dyn(), sorter),
-            num_seq_fallback: 2_400,
-            func,
-        }
-    }
-}
-
-impl<'a, T: Aabb, SO: NodeHandler<T>> CollidingPairsBuilderApi<'a, T, SO>
-    for TreeInner<Node<'a, T>, SO>
-{
-    fn vistr_mut<'b>(&'b mut self) -> VistrMutPin<'b, tree::node::Node<'a, T>> {
-        TreeInner::vistr_mut(self)
-    }
-
-    fn sorter(&mut self) -> SO {
-        TreeInner::sorter(self)
-    }
-}
+const SEQ_FALLBACK_DEFAULT: usize = 2_400;
 
 #[must_use]
-pub struct CollidingPairsBuilder<'a, 'b, T: Aabb, SO, F> {
-    pub vis: CollVis<'a, 'b, T, SO>,
+pub struct CollidingPairsBuilder<'a, 'b, T: Aabb, SO: NodeHandler<T>> {
+    vis: CollVis<'a, 'b, T>,
     pub num_seq_fallback: usize,
-    pub func: F,
+    pub handler: SO,
 }
 
-impl<'a, 'b, T: Aabb, SO: NodeHandler<T>, F: FnMut(AabbPin<&mut T>, AabbPin<&mut T>)>
-    CollidingPairsBuilder<'a, 'b, T, SO, F>
-{
+impl<'a, 'b, T: Aabb, SO: NodeHandler<T>> CollidingPairsBuilder<'a, 'b, T, SO> {
+    pub fn new(tree: &'b mut TreeInner<Node<'a, T>, SO::Sorter>, handler: SO) -> Self {
+        CollidingPairsBuilder {
+            vis: CollVis::new(tree.vistr_mut(), default_axis().to_dyn()),
+            num_seq_fallback: SEQ_FALLBACK_DEFAULT,
+            handler,
+        }
+    }
     pub fn build(mut self) {
-        let mut prevec = PreVec::new();
-        self.vis.recurse_seq(&mut prevec, &mut self.func);
+        self.vis.recurse_seq(&mut self.handler);
     }
 
     #[cfg(feature = "rayon")]
-    pub fn build_par(self)
+    pub fn build_par(self) -> SO
     where
         T: Send,
         T::Num: Send,
-        F: Clone + Send,
+        SO: Splitter + Send,
     {
         ///
         /// height_seq_fallback: if a subtree has this height, it will be processed as one unit sequentially.
         ///
         pub fn recurse_par<T: Aabb, N: NodeHandler<T>>(
-            vistr: CollVis<T, N>,
-            prevec: &mut PreVec,
+            vistr: CollVis<T>,
+            mut handler: N,
             num_seq_fallback: usize,
-            mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>) + Clone + Send,
-        ) where
+        ) -> N
+        where
             T: Send,
             T::Num: Send,
+            N: Splitter + Send,
         {
             if vistr.num_elem() <= num_seq_fallback {
-                vistr.recurse_seq(prevec, &mut func);
+                vistr.recurse_seq(&mut handler);
+                handler
             } else {
-                let func2 = func.clone();
-                let (n, rest) = vistr.collide_and_next(prevec, &mut func);
+                let (mut h1, h2) = handler.div();
+                let (n, rest) = vistr.collide_and_next(&mut h1);
                 if let Some([left, right]) = rest {
-                    rayon::join(
+                    let (h1, h2) = rayon::join(
                         || {
-                            let (prevec, func) = n.finish();
-                            recurse_par(left, prevec, num_seq_fallback, func.clone());
-                        },
-                        || {
-                            let mut prevec = PreVec::new();
-                            recurse_par(right, &mut prevec, num_seq_fallback, func2);
+                            n.finish(&mut h1);
+                            recurse_par(left, h1, num_seq_fallback)
                         },
+                        || recurse_par(right, h2, num_seq_fallback),
                     );
+                    h1.add(h2)
                 } else {
-                    let _ = n.finish();
+                    let _ = n.finish(&mut h1);
+                    h1
                 }
             }
         }
 
-        let mut prevec = PreVec::new();
-
-        //TODO best level define somewhere?
-        recurse_par(self.vis, &mut prevec, self.num_seq_fallback, self.func)
+        recurse_par(self.vis, self.handler, self.num_seq_fallback)
     }
 
     pub fn build_with_splitter<SS: Splitter>(mut self, splitter: SS) -> SS {
         pub fn recurse_seq_splitter<T: Aabb, S: NodeHandler<T>, SS: Splitter>(
-            vistr: CollVis<T, S>,
+            vistr: CollVis<T>,
             splitter: SS,
-            prevec: &mut PreVec,
-            func: &mut impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>),
+            func: &mut S,
         ) -> SS {
-            let (n, rest) = vistr.collide_and_next(prevec, func);
+            let (n, rest) = vistr.collide_and_next(func);
 
             if let Some([left, right]) = rest {
                 let (s1, s2) = splitter.div();
-                n.finish();
-                let al = recurse_seq_splitter(left, s1, prevec, func);
-                let ar = recurse_seq_splitter(right, s2, prevec, func);
+                n.finish(func);
+                let al = recurse_seq_splitter(left, s1, func);
+                let ar = recurse_seq_splitter(right, s2, func);
                 al.add(ar)
             } else {
-                n.finish();
+                n.finish(func);
                 splitter
             }
         }
-        let mut prevec = PreVec::new();
-        recurse_seq_splitter(self.vis, splitter, &mut prevec, &mut self.func)
+        recurse_seq_splitter(self.vis, splitter, &mut self.handler)
     }
 
+    /*
     #[cfg(feature = "rayon")]
     pub fn build_with_splitter_par<SS: Splitter + Send>(self, splitter: SS) -> SS
     where
         T: Send,
         T::Num: Send,
-        F: Clone + Send,
+        SO: Clone + Send,
     {
         ///
         /// height_seq_fallback: if a subtree has this height, it will be processed as one unit sequentially.
         ///
         pub fn recurse_par_splitter<T: Aabb, N: NodeHandler<T>, S: Splitter + Send>(
-            vistr: CollVis<T, N>,
-            prevec: &mut PreVec,
+            vistr: CollVis<T>,
             num_seq_fallback: usize,
-            mut func: impl FnMut(AabbPin<&mut T>, AabbPin<&mut T>) + Clone + Send,
+            mut func: N,
             splitter: S,
         ) -> S
         where
             T: Send,
             T::Num: Send,
+            N: Clone + Send,
         {
             if vistr.num_elem() <= num_seq_fallback {
-                vistr.recurse_seq(prevec, &mut func);
+                vistr.recurse_seq(&mut func);
                 splitter
             } else {
                 let func2 = func.clone();
-                let (n, rest) = vistr.collide_and_next(prevec, &mut func);
+                let (n, rest) = vistr.collide_and_next(&mut func);
                 if let Some([left, right]) = rest {
                     let (s1, s2) = splitter.div();
 
                     let (s1, s2) = rayon::join(
                         || {
-                            let (prevec, func) = n.finish();
-                            recurse_par_splitter(left, prevec, num_seq_fallback, func.clone(), s1)
-                        },
-                        || {
-                            let mut prevec = PreVec::new();
-                            recurse_par_splitter(right, &mut prevec, num_seq_fallback, func2, s2)
+                            n.finish(&mut func);
+                            recurse_par_splitter(left, num_seq_fallback, func, s1)
                         },
+                        || recurse_par_splitter(right, num_seq_fallback, func2, s2),
                     );
 
                     s1.add(s2)
                 } else {
-                    let _ = n.finish();
+                    let _ = n.finish(&mut func);
                     splitter
                 }
             }
         }
-        let mut prevec = PreVec::new();
-        recurse_par_splitter(
-            self.vis,
-            &mut prevec,
-            self.num_seq_fallback,
-            self.func,
-            splitter,
-        )
+        recurse_par_splitter(self.vis, self.num_seq_fallback, self.handler, splitter)
     }
+    */
 }
